@@ -4,10 +4,10 @@ interface using the input and output types.
 """
 
 import gradio
-from gradio.inputs import InputComponent
-from gradio.outputs import OutputComponent
+from gradio.inputs import InputComponent, get_input_instance
+from gradio.outputs import OutputComponent, get_output_instance
 from gradio import networking, strings, utils
-from gradio.interpretation import quantify_difference_in_label
+from gradio.interpretation import quantify_difference_in_label, get_regression_or_classification_value
 from gradio.external import load_interface
 from gradio import encryptor
 import pkg_resources
@@ -65,9 +65,9 @@ class Interface:
 
     def __init__(self, fn, inputs=None, outputs=None, verbose=False, examples=None,
                  examples_per_page=10, live=False,
-                 layout="horizontal", show_input=True, show_output=True,
-                 capture_session=False, interpretation=None, theme=None, repeat_outputs_per_model=True,
-                 title=None, description=None, article=None, thumbnail=None, 
+                 layout="unaligned", show_input=True, show_output=True,
+                 capture_session=False, interpretation=None, num_shap=2.0, theme=None, repeat_outputs_per_model=True,
+                 title=None, description=None, article=None, thumbnail=None,  
                  css=None, server_port=None, server_name=networking.LOCALHOST_NAME, height=500, width=900,
                  allow_screenshot=True, allow_flagging=True, flagging_options=None, encrypt=False,
                  show_tips=False, embedding=None, flagging_dir="flagged", analytics_enabled=True):
@@ -78,17 +78,18 @@ class Interface:
         inputs (Union[str, List[Union[str, InputComponent]]]): a single Gradio input component, or list of Gradio input components. Components can either be passed as instantiated objects, or referred to by their string shortcuts. The number of input components should match the number of parameters in fn.
         outputs (Union[str, List[Union[str, OutputComponent]]]): a single Gradio output component, or list of Gradio output components. Components can either be passed as instantiated objects, or referred to by their string shortcuts. The number of output components should match the number of values returned by fn.
         verbose (bool): whether to print detailed information during launch.
-        examples (Union[List[List[Any]], str]): sample inputs for the function; if provided, appears below the UI components and can be used to populate the interface. Should be nested list, in which the outer list consists of samples and each inner list consists of an input corresponding to each input component. A string path to a directory of examples can also be provided. If there are multiple input components, a log.csv file must be present in the directory to link corresponding inputs.
+        examples (Union[List[List[Any]], str]): sample inputs for the function; if provided, appears below the UI components and can be used to populate the interface. Should be nested list, in which the outer list consists of samples and each inner list consists of an input corresponding to each input component. A string path to a directory of examples can also be provided. If there are multiple input components and a directory is provided, a log.csv file must be present in the directory to link corresponding inputs.
         examples_per_page (int): If examples are provided, how many to display per page.
         live (bool): whether the interface should automatically reload on change.
         layout (str): Layout of input and output panels. "horizontal" arranges them as two columns of equal height, "unaligned" arranges them as two columns of unequal height, and "vertical" arranges them vertically.
         capture_session (bool): if True, captures the default graph and session (needed for Tensorflow 1.x)
         interpretation (Union[Callable, str]): function that provides interpretation explaining prediction output. Pass "default" to use built-in interpreter. 
+        num_shap (float): a multiplier that determines how many examples are computed for shap-based interpretation. Increasing this value will increase shap runtime, but improve results.
         title (str): a title for the interface; if provided, appears above the input and output components.
         description (str): a description for the interface; if provided, appears above the input and output components.
         article (str): an expanded article explaining the interface; if provided, appears below the input and output components. Accepts Markdown and HTML content.
         thumbnail (str): path to image or src to use as display picture for models listed in gradio.app/hub
-        theme (str): Theme to use - one of "dark", "default", or "huggingface".
+        theme (str): Theme to use - one of "default", "compact" or "huggingface".
         css (str): custom css or path to custom css file to use with interface.
         server_port (int): will start gradio app on this port (if available) 
         server_name (str): to make app accessible on local network set to "0.0.0.0".
@@ -99,28 +100,6 @@ class Interface:
         flagging_dir (str): what to name the dir where flagged data is stored.
         show_tips (bool): if True, will occasionally show tips about new Gradio features
         """
-        def get_input_instance(iface):
-            if isinstance(iface, str):
-                shortcut = InputComponent.get_all_shortcut_implementations()[iface]
-                return shortcut[0](**shortcut[1])
-            elif isinstance(iface, InputComponent):
-                return iface
-            else:
-                raise ValueError("Input interface must be of type `str` or "
-                                 "`InputComponent` but is {}".format(iface))
-
-        def get_output_instance(iface):
-            if isinstance(iface, str):
-                shortcut = OutputComponent.get_all_shortcut_implementations()[iface]
-                return shortcut[0](**shortcut[1])
-            elif isinstance(iface, OutputComponent):
-                return iface
-            else:
-                raise ValueError(
-                    "Output interface must be of type `str` or "
-                    "`OutputComponent`"
-                )
-
         if not isinstance(fn, list):
             fn = [fn]
         if isinstance(inputs, list):
@@ -134,6 +113,14 @@ class Interface:
 
         if repeat_outputs_per_model:
             self.output_components *= len(fn)
+
+        if interpretation is None or isinstance(interpretation, list) or callable(interpretation):
+            self.interpretation = interpretation 
+        elif isinstance(interpretation, str):
+            self.interpretation = [interpretation.lower() for _ in self.input_components]
+        else:
+            raise ValueError("Invalid value for parameter: interpretation")         
+            
         self.predict = fn
         self.function_names = [func.__name__ for func in fn]
         self.__name__ = ", ".join(self.function_names)
@@ -145,7 +132,6 @@ class Interface:
         self.show_output = show_output
         self.flag_hash = random.getrandbits(32)
         self.capture_session = capture_session
-        self.interpretation = interpretation            
         self.session = None
         self.server_name = server_name
         self.title = title
@@ -167,6 +153,7 @@ class Interface:
             self.examples = examples
         else:
             raise ValueError("Examples argument must either be a directory or a nested list, where each sublist represents a set of inputs.")
+        self.num_shap = num_shap
         self.examples_per_page = examples_per_page
         self.server_port = server_port
         self.simple_server = None
@@ -183,6 +170,7 @@ class Interface:
         self.local_url = None
         self.embedding = embedding
         self.show_tips = show_tips
+        self.requires_permissions = any([component.requires_permissions for component in self.input_components])
 
         data = {'fn': fn,
                 'inputs': inputs,
@@ -251,6 +239,7 @@ class Interface:
             "description": self.description,
             "article": self.article,
             "theme": self.theme,
+            "css": self.css,
             "thumbnail": self.thumbnail,
             "allow_screenshot": self.allow_screenshot,
             "allow_flagging": self.allow_flagging,
@@ -268,7 +257,7 @@ class Interface:
                 function_index = i // outputs_per_function
                 component_index = i - function_index * outputs_per_function
                 ret_name = "Output " + str(component_index + 1) if outputs_per_function > 1 else "Output"
-                if not iface["label"]:
+                if iface["label"] is None:
                     iface["label"] = ret_name
                 if len(self.predict) > 1:
                     iface["label"] = self.function_names[function_index].replace("_", " ") + ": " + iface["label"]
@@ -338,7 +327,7 @@ class Interface:
                            for i, input_component in enumerate(self.input_components)]
         predictions, durations = self.run_prediction(processed_input, return_duration=True)
         processed_output = [output_component.postprocess(
-            predictions[i]) for i, output_component in enumerate(self.output_components)]
+            predictions[i]) if predictions[i] is not None else None for i, output_component in enumerate(self.output_components)]
         return processed_output, durations
     
     def embed(self, processed_input):
@@ -355,35 +344,87 @@ class Interface:
         interpretation for a certain set of UI component types, as well as the custom interpretation case.
         :param raw_input: a list of raw inputs to apply the interpretation(s) on.
         """
-        if self.interpretation == "default":
+        if isinstance(self.interpretation, list):  # Either "default" or "shap"
             processed_input = [input_component.preprocess(raw_input[i])
                             for i, input_component in enumerate(self.input_components)]
             original_output = self.run_prediction(processed_input)
             scores, alternative_outputs = [], []
-            for i, x in enumerate(raw_input):
-                input_component = self.input_components[i]
-                neighbor_raw_input = list(raw_input)
-                neighbor_values, interpret_kwargs, interpret_by_removal = input_component.get_interpretation_neighbors(x)
-                interface_scores = []
-                alternative_output = []
-                for neighbor_input in neighbor_values:
-                    neighbor_raw_input[i] = neighbor_input
-                    processed_neighbor_input = [input_component.preprocess(neighbor_raw_input[i])
-                                    for i, input_component in enumerate(self.input_components)]
-                    neighbor_output = self.run_prediction(processed_neighbor_input)
-                    processed_neighbor_output = [output_component.postprocess(
-                        neighbor_output[i]) for i, output_component in enumerate(self.output_components)]
+            for i, (x, interp) in enumerate(zip(raw_input, self.interpretation)):
+                print(i, interp)
+                if interp=="default":
+                    input_component = self.input_components[i]
+                    neighbor_raw_input = list(raw_input)
+                    if input_component.interpret_by_tokens:
+                        tokens, neighbor_values, masks =  input_component.tokenize(x)
+                        interface_scores = []
+                        alternative_output = []
+                        for neighbor_input in neighbor_values:
+                            neighbor_raw_input[i] = neighbor_input
+                            processed_neighbor_input = [input_component.preprocess(neighbor_raw_input[i])
+                                            for i, input_component in enumerate(self.input_components)]
+                            neighbor_output = self.run_prediction(processed_neighbor_input)
+                            processed_neighbor_output = [output_component.postprocess(
+                                neighbor_output[i]) for i, output_component in enumerate(self.output_components)]
 
-                    alternative_output.append(processed_neighbor_output)
-                    interface_scores.append(quantify_difference_in_label(self, original_output, neighbor_output))
-                alternative_outputs.append(alternative_output)
-                if not interpret_by_removal:
-                    interface_scores = [-score for score in interface_scores]
-                scores.append(
-                    input_component.get_interpretation_scores(
-                        raw_input[i], neighbor_values, interface_scores, **interpret_kwargs))
+                            alternative_output.append(processed_neighbor_output)
+                            interface_scores.append(quantify_difference_in_label(self, original_output, neighbor_output))
+                        alternative_outputs.append(alternative_output)
+                        scores.append(
+                            input_component.get_interpretation_scores(
+                                raw_input[i], neighbor_values, interface_scores, masks=masks, tokens=tokens))                
+                    else:
+                        neighbor_values, interpret_kwargs = input_component.get_interpretation_neighbors(x)
+                        interface_scores = []
+                        alternative_output = []
+                        for neighbor_input in neighbor_values:
+                            neighbor_raw_input[i] = neighbor_input
+                            processed_neighbor_input = [input_component.preprocess(neighbor_raw_input[i])
+                                            for i, input_component in enumerate(self.input_components)]
+                            neighbor_output = self.run_prediction(processed_neighbor_input)
+                            processed_neighbor_output = [output_component.postprocess(
+                                neighbor_output[i]) for i, output_component in enumerate(self.output_components)]
+
+                            alternative_output.append(processed_neighbor_output)
+                            interface_scores.append(quantify_difference_in_label(self, original_output, neighbor_output))
+                        alternative_outputs.append(alternative_output)
+                        interface_scores = [-score for score in interface_scores]
+                        scores.append(
+                            input_component.get_interpretation_scores(
+                                raw_input[i], neighbor_values, interface_scores, **interpret_kwargs))
+                elif interp == "shap":
+                    try:
+                        import shap
+                    except (ImportError, ModuleNotFoundError):
+                        raise ValueError("The package `shap` is required for this interpretation method. Try: `pip install shap`")
+                    input_component = self.input_components[i]
+                    if not(input_component.interpret_by_tokens):
+                        raise ValueError("Input component {} does not support `shap` interpretation".format(input_component))
+                    
+                    tokens, _, masks = input_component.tokenize(x)
+                    
+                    def get_masked_prediction(binary_mask):  # construct a masked version of the input
+                        masked_xs = input_component.get_masked_inputs(tokens, binary_mask)  
+                        preds = []
+                        for masked_x in masked_xs:
+                            processed_masked_input = copy.deepcopy(processed_input) 
+                            processed_masked_input[i] = input_component.preprocess(masked_x) 
+                            new_output = self.run_prediction(processed_masked_input)
+                            pred = get_regression_or_classification_value(self, original_output, new_output)
+                            preds.append(pred)
+                        return np.array(preds)
+
+                    num_total_segments = len(tokens) 
+                    explainer = shap.KernelExplainer(get_masked_prediction, np.zeros((1, num_total_segments)))
+                    shap_values = explainer.shap_values(np.ones((1, num_total_segments)), nsamples=int(self.num_shap*num_total_segments), silent=True)
+                    scores.append(input_component.get_interpretation_scores(raw_input[i], None, shap_values[0], masks=masks, tokens=tokens))
+                    alternative_outputs.append([])
+                elif interp is None:
+                    scores.append(None)
+                    alternative_outputs.append([])
+                else:
+                    raise ValueError("Uknown intepretation method: {}".format(interp))
             return scores, alternative_outputs
-        else:
+        else:  # custom interpretation function
             processed_input = [input_component.preprocess(raw_input[i])
                                for i, input_component in enumerate(self.input_components)]
             interpreter = self.interpretation
@@ -411,8 +452,8 @@ class Interface:
 
     def run_until_interrupted(self, thread, path_to_local_server):
         try:
-            while 1:
-                pass
+            while True:
+                time.sleep(0.5)
         except (KeyboardInterrupt, OSError):
             print("Keyboard interruption in main thread... closing server.")
             thread.keep_running = False
@@ -441,7 +482,7 @@ class Interface:
         inbrowser (bool): whether to automatically launch the interface in a new tab on the default browser.
         share (bool): whether to create a publicly shareable link from your computer for the interface.
         debug (bool): if True, and the interface was launched from Google Colab, prints the errors in the cell output.
-        auth (Union[Tuple[str, str], List[Tuple[str, str]]]): If provided, username and password (or list of username-password tuples) required to access interface.
+        auth (Callable, Union[Tuple[str, str], List[Tuple[str, str]]]): If provided, username and password (or list of username-password tuples) required to access interface. Can also provide function that takes username and password and returns True if valid login.
         auth_message (str): If provided, HTML message provided on login page.
         Returns:
         app (flask.Flask): Flask app object
@@ -454,7 +495,7 @@ class Interface:
         # Set up local flask server
         config = self.get_config_file()
         self.config = config
-        if auth and not isinstance(auth[0], tuple) and not isinstance(auth[0], list):
+        if auth and not callable(auth) and not isinstance(auth[0], tuple) and not isinstance(auth[0], list):
             auth = [auth]
         self.auth = auth
         self.auth_message = auth_message
@@ -486,6 +527,8 @@ class Interface:
                     print(strings.en["COLAB_DEBUG_FALSE"])
         else:
             print(strings.en["RUNNING_LOCALLY"].format(path_to_local_server))
+        if is_colab and self.requires_permissions:
+            print(strings.en["MEDIA_PERMISSIONS_IN_COLAB"])
 
         if private_endpoint is not None:
             share = True
@@ -493,7 +536,9 @@ class Interface:
         self.share = share
 
         if share:
-            if not private_endpoint:
+            if private_endpoint:
+                print(strings.en["PRIVATE_LINK_MESSAGE"])
+            else:
                 print(strings.en["SHARE_LINK_MESSAGE"])
             try:
                 share_url = networking.setup_tunnel(server_port, private_endpoint)
@@ -548,7 +593,7 @@ class Interface:
         return app, path_to_local_server, share_url
 
 
-    def integrate(self, comet_ml=None):
+    def integrate(self, comet_ml=None, wandb=None, mlflow=None):
         if comet_ml is not None:
             comet_ml.log_other("Created from", "Gradio")
             if self.share_url is not None:
@@ -557,21 +602,25 @@ class Interface:
             else:
                 comet_ml.log_text("gradio: " + self.local_url)
                 comet_ml.end()
+        if wandb is not None:
+            if self.share_url is not None:
+                wandb.log({"Gradio panel": wandb.Html('<iframe src="' + self.share_url + '" width="' + str(self.width) + '" height="' + str(self.height) + '" frameBorder="0"></iframe>')})
+            else:
+                print("The WandB integration requires you to `launch(share=True)` first.")
+        if mlflow is not None:
+            if self.share_url is not None:
+                mlflow.log_param("Gradio Interface Share Link",
+                                 self.share_url)
+            else:
+                mlflow.log_param("Gradio Interface Local Link",
+                                 self.local_url)
+
     
 
 def show_tip(io):
-    if not(io.show_tips):
+    if not(io.show_tips) or random.random() < 0.5:  # Only show tip every other use.
         return
-    if random.random() < 0.8:  # Only show tips once every 5 uses
-        return
-    relevant_tips = []
-    if io.interpretation is None:
-        relevant_tips.append(strings.en["TIP_INTERPRETATION"])
-    if io.embedding is None and not(io.examples is None) and len(io.examples)>4:
-        relevant_tips.append(strings.en["TIP_EMBEDDING"])
-    if len(relevant_tips)==0:
-        return
-    print(random.choice(relevant_tips))
+    print(random.choice(strings.en.TIPS))
 
 def launch_counter():
     try:
