@@ -441,21 +441,30 @@ class Dataframe(OutputComponent):
     Output type: Union[pandas.DataFrame, numpy.array, List[Union[str, float]], List[List[Union[str, float]]]]
     """
 
-    def __init__(self, headers=None, type="auto", label=None):
+    def __init__(self, headers=None, max_rows=20, max_cols=None, overflow_row_behaviour="paginate", type="auto", label=None):
         '''
         Parameters:
-        headers (List[str]): Header names to dataframe.
+        headers (List[str]): Header names to dataframe. Only applicable if type is "numpy" or "array".
+        max_rows (int): Maximum number of rows to display at once. Set to None for infinite. 
+        max_cols (int): Maximum number of columns to display at once. Set to None for infinite.
+        overflow_row_behaviour (str): If set to "paginate", will create pages for overflow rows. If set to "show_ends", will show initial and final rows and truncate middle rows. 
         type (str): Type of value to be passed to component. "pandas" for pandas dataframe, "numpy" for numpy array, or "array" for Python array, "auto" detects return type.
         label (str): component name in interface.
         '''
-        self.type = type
         self.headers = headers
+        self.max_rows = max_rows
+        self.max_cols = max_cols
+        self.overflow_row_behaviour = overflow_row_behaviour
+        self.type = type
         super().__init__(label)
 
 
     def get_template_context(self):
         return {
             "headers": self.headers,
+            "max_rows": self.max_rows,
+            "max_cols": self.max_cols,
+            "overflow_row_behaviour": self.overflow_row_behaviour,
             **super().get_template_context()
         }
 
@@ -496,4 +505,64 @@ class Dataframe(OutputComponent):
         return json.dumps(data["data"])
 
     def restore_flagged(self, data):
-        return json.loads(data)
\ No newline at end of file
+        return json.loads(data)
+
+
+class Carousel(OutputComponent):
+    """
+    Component displays a set of output components that can be scrolled through.
+    Output type: List[List[Any]]
+    """
+
+    def __init__(self, components, label=None):
+        '''
+        Parameters:
+        components (Union[List[OutputComponent], OutputComponent]): Classes of component(s) that will be scrolled through.
+        label (str): component name in interface.
+        '''
+        if not isinstance(components, list):
+            components = [components]
+        self.components = [get_output_instance(component) for component in components]
+        super().__init__(label)
+
+
+    def get_template_context(self):
+        return {
+            "components": [component.get_template_context() for component in self.components],
+            **super().get_template_context()
+        }
+
+    def postprocess(self, y):
+        if isinstance(y, list):
+            if len(y) != 0 and not isinstance(y[0], list):
+                y = [[z] for z in y]
+            output = []
+            for row in y:
+                output_row = []
+                for i, cell in enumerate(row):
+                    output_row.append(self.components[i].postprocess(cell))
+                output.append(output_row)
+            return output
+        else:
+            raise ValueError("Unknown type. Please provide a list for the Carousel.")
+
+    def save_flagged(self, dir, label, data, encryption_key):
+        return json.dumps([
+            [
+                component.save_flagged(dir, f"{label}_{j}", data[i][j], encryption_key)
+                for j, component in enumerate(self.components)
+            ] for i, sample in enumerate(data)])
+
+
+
+def get_output_instance(iface):
+    if isinstance(iface, str):
+        shortcut = OutputComponent.get_all_shortcut_implementations()[iface]
+        return shortcut[0](**shortcut[1])
+    elif isinstance(iface, OutputComponent):
+        return iface
+    else:
+        raise ValueError(
+            "Output interface must be of type `str` or "
+            "`OutputComponent`"
+        )
